"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrpcBrowser = void 0;
const event_name_1 = require("../protocol/browser/event-name");
const status_msg_menu_changed_1 = require("../protocol/browser/event/status-msg-menu-changed");
const busy_changed_1 = require("../protocol/event/busy-changed");
const state_changed_1 = require("../protocol/event/state-changed");
const control_with_properties_abstract_1 = require("./control-with-properties-abstract");
const list_item_1 = require("./list-item");
const get_data_1 = require("../protocol/browser/packet/get-data");
const back_1 = require("../protocol/browser/packet/back");
const select_1 = require("../protocol/browser/packet/select");
const list_function_1 = require("./list-function");
const send_1 = require("../protocol/browser/packet/send");
class CrpcBrowser extends control_with_properties_abstract_1.ControlWithPropertiesAbstract {
    constructor(_handle, _uuid, _instanceName, _protocol) {
        super(_handle, _instanceName, _protocol);
        this._automaticListItemRetrievalEnabled = false;
        this.version$ = this._createPropertySubject('Version', 0);
        this.maxReqItems$ = this._createPropertySubject('MaxReqItems', 0);
        this.level$ = this._createPropertySubject('Level', 0);
        this.itemCnt$ = this._createPropertySubject('ItemCnt', 0, control_with_properties_abstract_1.PropertyUpdateType.VALUE_RECEIVE);
        this.title$ = this._createPropertySubject('Title', '');
        this.subtitle$ = this._createPropertySubject('Subtitle', '');
        this.listSpecificFunctions$ = this._createPropertySubject('ListSpecificFunctions', []);
        this.isMenuAvailable$ = this._createPropertySubject('IsMenuAvailable', false);
        this.statusMsgMenu$ = this._createPropertySubject('StatusMsgMenu', {});
        this.instance$ = this._createPropertySubject('Instance', 0);
        this.items$ = this._createPropertySubject('ListItems', [], control_with_properties_abstract_1.PropertyUpdateType.VALUE_CHANGE, () => this._enableAutomaticListRetrieval());
        this.functions$ = this._createPropertySubject('ListFunctions', [], control_with_properties_abstract_1.PropertyUpdateType.VALUE_CHANGE, () => this._enableAutomaticListSpecificFunctions());
    }
    initialize() {
        this._registerEvent(event_name_1.EventName.BusyChanged).catch((e) => console.error('Register BusyChanged failed.', e));
        this._registerEvent(event_name_1.EventName.ClearChanged).catch((e) => console.error('Register ClearChanged failed.', e));
        this._registerEvent(event_name_1.EventName.ListChanged).catch((e) => console.error('Register ListChanged failed.', e));
        this._registerEvent(event_name_1.EventName.StateChanged).catch((e) => console.error('Register StateChanged failed.', e));
        this._registerEvent(event_name_1.EventName.StatusMsgMenuChanged).catch((e) => console.error('Register StatusMsgMenuChanged failed.', e));
    }
    _handleEvent(event, properties) {
        switch (event) {
            case event_name_1.EventName.BusyChanged:
                this._onBusyChanged(Object.assign(new busy_changed_1.BusyChanged(), properties || {}));
                break;
            case event_name_1.EventName.ClearChanged:
                // TODO: ClearChanged
                break;
            case event_name_1.EventName.ListChanged:
                // TODO: ListChanged
                break;
            case event_name_1.EventName.StateChanged:
                this._onStateChanged(Object.assign(new state_changed_1.StateChanged(), properties));
                break;
            case event_name_1.EventName.StatusMsgMenuChanged:
                this._onDialogChanged(Object.assign(new status_msg_menu_changed_1.StatusMsgMenuChanged(), properties));
                break;
        }
    }
    _enableAutomaticListRetrieval() {
        this.itemCnt$.subscribe({
            next: async (itemCount) => {
                if (itemCount === 0) {
                    this._updateLocalProperty('ListItems', []);
                    return;
                }
                const response = await this._protocol.send(new get_data_1.GetData(this._instanceName, itemCount)).catch((e) => console.error('Failed to retrieve browser list items!', e));
                const items = (response.result || []).map((rawItem, index) => {
                    return list_item_1.ListItem.fromCrpc(rawItem, (index + 1), () => this._selectItemByIndex(index + 1));
                });
                this._updateLocalProperty('ListItems', items);
            }
        });
    }
    _enableAutomaticListSpecificFunctions() {
        this.listSpecificFunctions$.subscribe({
            next: async (functions) => {
                if ((functions === null || functions === void 0 ? void 0 : functions.length) === 0) {
                    this._updateLocalProperty('ListFunctions', []);
                    return;
                }
                const items = functions.map((listFunction) => new list_function_1.ListFunction(listFunction, () => this._executeListSpecificFunction(listFunction)));
                this._updateLocalProperty('ListFunctions', items);
            }
        });
    }
    _executeListSpecificFunction(name) {
        this._protocol.send(Object.assign(new send_1.Send(this._instanceName), {
            method: name
        }));
    }
    _selectItemByIndex(index) {
        this._protocol.send(new select_1.Select(this._instanceName, index));
    }
    back() {
        this._protocol.send(new back_1.Back(this._instanceName));
    }
}
exports.CrpcBrowser = CrpcBrowser;
