"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrpcPlayer = void 0;
const busy_changed_1 = require("../protocol/event/busy-changed");
const state_changed_1 = require("../protocol/event/state-changed");
const event_name_1 = require("../protocol/player/event-name");
const status_msg_changed_1 = require("../protocol/player/event/status-msg-changed");
const fast_forward_1 = require("../protocol/player/packet/fast-forward");
const next_category_1 = require("../protocol/player/packet/next-category");
const next_track_1 = require("../protocol/player/packet/next-track");
const pause_1 = require("../protocol/player/packet/pause");
const play_1 = require("../protocol/player/packet/play");
const previous_category_1 = require("../protocol/player/packet/previous-category");
const previous_track_1 = require("../protocol/player/packet/previous-track");
const repeat_1 = require("../protocol/player/packet/repeat");
const rewind_1 = require("../protocol/player/packet/rewind");
const seek_1 = require("../protocol/player/packet/seek");
const shuffle_1 = require("../protocol/player/packet/shuffle");
const star_1 = require("../protocol/player/packet/star");
const thumbs_down_1 = require("../protocol/player/packet/thumbs-down");
const thumbs_up_1 = require("../protocol/player/packet/thumbs-up");
const property_1 = require("../protocol/player/property");
const rating_1 = require("../protocol/player/rating");
const control_with_properties_abstract_1 = require("./control-with-properties-abstract");
class CrpcPlayer extends control_with_properties_abstract_1.ControlWithPropertiesAbstract {
    constructor(_handle, _uuid, _instanceName, _protocol) {
        super(_handle, _instanceName, _protocol);
        this.textLines$ = this._createPropertySubject('TextLines', []);
        this.actionsSupported$ = this._createPropertySubject('ActionsSupported', []);
        this.actionsAvailable$ = this._createPropertySubject('ActionsAvailable', []);
        this.progressBar$ = this._createPropertySubject('ProgressBar', false);
        this.elapsedSec$ = this._createPropertySubject('ElapsedSec', 0);
        this.trackSec$ = this._createPropertySubject('TrackSec', 0);
        // TODO: Create num for streamstates: 'streaming'
        this.streamState$ = this._createPropertySubject('StreamState', '');
        this.trackNum$ = this._createPropertySubject('TrackNum', 0);
        this.trackCnt$ = this._createPropertySubject('TrackCnt', 0);
        // TODO: create enum for playerstates: playing | paused | stopped
        this.playerState$ = this._createPropertySubject('PlayerState', '');
        // TODO: Check what playerIcon is, i'm guessing it's a crestron icon number.
        this.playerIcon$ = this._createPropertySubject('PlayerIcon', 0);
        this.playerName$ = this._createPropertySubject('PlayerName', '');
        this.playerIconUrl$ = this._createPropertySubject('PlayerIconURL', '');
        // TODO: Create enum for media types: 'audio'
        this.mediaType$ = this._createPropertySubject('MediaType', '');
        this.instance$ = this._createPropertySubject('Instance', 0);
        this.albumArt$ = this._createPropertySubject('AlbumArt', '');
        this.albumArtUrl$ = this._createPropertySubject('AlbumArtUrl', '');
        this.albumArtUrlNat$ = this._createPropertySubject('AlbumArtUrlNAT', '');
        this.stationName$ = this._createPropertySubject('StationName', '');
        this.album$ = this._createPropertySubject('Album', '');
        this.genre$ = this._createPropertySubject('Genre', '');
        this.artist$ = this._createPropertySubject('Artist', '');
        this.title$ = this._createPropertySubject('Title', '');
        this.language$ = this._createPropertySubject('Language', '');
        this.rewindSpeed$ = this._createPropertySubject('RewindSpeed', 0);
        this.providerName$ = this._createPropertySubject('ProviderName', '');
        this.fastForwardSpeed$ = this._createPropertySubject('FfwdSpeed', 0);
        this.nextTitle$ = this._createPropertySubject('NextTitle', '');
        this.mediaReady$ = this._createPropertySubject('MediaReady', false);
        // TODO: Create enum for shufflestate: '0'(Off)
        this.shuffleState$ = this._createPropertySubject('ShuffleState', 0);
        // TODO: Create enum for repeatstate: '0'(not supported?)
        this.repeatState$ = this._createPropertySubject('RepeatState', 0);
        this.rating$ = this._createPropertySubject('Rating', new rating_1.Rating());
        this.selectedId$ = this._createPropertySubject('SelectedId', '');
    }
    initialize() {
        this._registerEvent(event_name_1.EventName.BusyChanged).catch(() => console.error('Register BusyChanged failed.'));
        this._registerEvent(event_name_1.EventName.StateChangedByBrowseContext).catch(() => console.error('Register StateChangedByBrowseContext failed.'));
        this._registerEvent(event_name_1.EventName.StatusMsgChanged).catch(() => console.error('Register StatusMsgChanged failed.'));
        this._registerEvent(event_name_1.EventName.StateChanged).catch(() => console.error('Register StateChanged failed.'));
        this._getProperty(property_1.Property.PropertiesSupported).catch((e) => console.error('Get PropertiesSupported failed.', e));
        this._getProperty(property_1.Property.ActionsSupported).catch((e) => console.error('Get PropertiesSupported failed.', e));
        this._getProperty(property_1.Property.ActionsAvailable).catch((e) => console.error('Get PropertiesSupported failed.', e));
    }
    _handleEvent(event, properties) {
        switch (event) {
            case event_name_1.EventName.BusyChanged:
                this._onBusyChanged(Object.assign(new busy_changed_1.BusyChanged(), properties || {}));
                break;
            case event_name_1.EventName.StateChangedByBrowseContext:
                // TODO: Handle StateChangedByBrowseContext
                break;
            case event_name_1.EventName.StatusMsgChanged:
                this._onDialogChanged(Object.assign(new status_msg_changed_1.StatusMsgChanged(), properties || {}));
                break;
            case event_name_1.EventName.StateChanged:
                this._onStateChanged(Object.assign(new state_changed_1.StateChanged(), properties));
                break;
        }
    }
    play() {
        this._protocol.send(new play_1.Play(this._instanceName));
    }
    pause() {
        this._protocol.send(new pause_1.Pause(this._instanceName));
    }
    nextTrack() {
        this._protocol.send(new next_track_1.NextTrack(this._instanceName));
    }
    previousTrack() {
        this._protocol.send(new previous_track_1.PreviousTrack(this._instanceName));
    }
    nextCategory() {
        this._protocol.send(new next_category_1.NextCategory(this._instanceName));
    }
    previousCategory() {
        this._protocol.send(new previous_category_1.PreviousCategory(this._instanceName));
    }
    thumbsUp() {
        this._protocol.send(new thumbs_up_1.ThumbsUp(this._instanceName));
    }
    thumbsDown() {
        this._protocol.send(new thumbs_down_1.ThumbsDown(this._instanceName));
    }
    rewind() {
        this._protocol.send(new rewind_1.Rewind(this._instanceName));
    }
    fastForward() {
        this._protocol.send(new fast_forward_1.FastForward(this._instanceName));
    }
    shuffle() {
        this._protocol.send(new shuffle_1.Shuffle(this._instanceName));
    }
    repeat() {
        this._protocol.send(new repeat_1.Repeat(this._instanceName));
    }
    star() {
        this._protocol.send(new star_1.Star(this._instanceName));
    }
    seek(seconds) {
        this._protocol.send(new seek_1.Seek(this._instanceName, seconds));
    }
}
exports.CrpcPlayer = CrpcPlayer;
